
#ifndef __ATLWIN_H__
	#error CDxWindowImpl.h requires atlwin.h to be included first
#endif



//---------------------------------------------------------------------------------------
// Class:  CDxWindowImpl
// Desc:   Main window for DirectX applications.
//---------------------------------------------------------------------------------------

typedef CWinTraits<WS_OVERLAPPEDWINDOW,0> CDxAppWinTraits;

template <class T, class TBase = CWindow, class TWinTraits = CDxAppWinTraits>
class CDxWindowImpl : public CWindowImpl<T,TBase,TWinTraits>
{
public:
      

//---------------------------------------------------------------------------------------
// Function:  GetWndClassInfo
// Desc:      Creates and returns the CWndClassInfo object used by the WTL libraries to
//            register the window class.
//---------------------------------------------------------------------------------------

  static CWndClassInfo& GetWndClassInfo()
  {
	  static CWndClassInfo wc =
    {
      { sizeof(WNDCLASSEX), CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS, StartWindowProc,
		    0, 0, NULL, NULL, NULL, AtlGetStockBrush(BLACK_BRUSH), NULL,
        T::GetWndClassName(), NULL
      },
      NULL, NULL, IDC_ARROW, TRUE, 0, _T("")
    };

	  return wc;
  }


//---------------------------------------------------------------------------------------
// Function:  ActivateApp()
// Desc:      Override to handle window activation.
//---------------------------------------------------------------------------------------

  virtual void ActivateApp()
  {
  }


//---------------------------------------------------------------------------------------
// Function:  DeactivateApp()
// Desc:      Override to handle window deactivation.
//---------------------------------------------------------------------------------------

  virtual void DeactivateApp()
  {
  }


//---------------------------------------------------------------------------------------
// Function:  GetWndClassName
// Desc:      Returns a unique window class name.
//---------------------------------------------------------------------------------------

  static LPCTSTR GetWndClassName()
  {
    return _T("CDxWindowImpl");
  }


//---------------------------------------------------------------------------------------
// Function:  AllowMultipleInstances
// Desc:      Allows/Prevents multiple instances of this app.  Return false to prevent.
//---------------------------------------------------------------------------------------

  static BOOL AllowMultipleInstances()
  {
    return false;
  }


//-----------------------------------------------------------------------------------------
// Function:  PreviousInstanceFound
// Desc:      Checks for a previous instance of this app and returns true if one is found.
//            If AllowMultipleInstances() returns false an existing instance's window is
//            flashed.
//-----------------------------------------------------------------------------------------

  BOOL PreviousInstanceFound(LPCTSTR lpClassName, LPCTSTR lpWindowName)
  {
    HWND hwnd = FindWindow(lpClassName,lpWindowName);

    if(hwnd)
    {
      if(!T::AllowMultipleInstances())
      {
        FLASHWINFO flashinfo;

        flashinfo.cbSize = sizeof(flashinfo);
        flashinfo.hwnd = hwnd;
        flashinfo.dwFlags = FLASHW_ALL;
        flashinfo.uCount = 2;
        flashinfo.dwTimeout = 0;

        FlashWindowEx(&flashinfo);
      }
      return true;
    }

    return false;
  }


//-----------------------------------------------------------------------------------------
// Function:  Create
// Desc:      Delagates window creation to CWindowImpl and stores the window class styles
//            and position.  Returns NULL if a previous instance exists but are disallowed.
//-----------------------------------------------------------------------------------------

  HWND Create(HWND hWndParent, RECT& rcPos, LPCTSTR szWindowName = NULL,
	            DWORD dwStyle = 0, DWORD dwExStyle = 0,
	            UINT nID = 0, LPVOID lpCreateParam = NULL)
  {
    if(PreviousInstanceFound(GetWndClassInfo().m_wc.lpszClassName,szWindowName) && !AllowMultipleInstances())
      return NULL;

    HWND hwnd = CWindowImpl<T,TBase,TWinTraits>::Create(hWndParent,rcPos,szWindowName,dwStyle,dwExStyle,nID,lpCreateParam);

    if(!hwnd)
      return NULL;

    //  Save the styles and position of the window
    wndStyles = GetWindowLong(GWL_STYLE);
    wndExStyles = GetWindowLong(GWL_EXSTYLE);
    GetWindowRect(&wndRect);

    return hwnd;
  }


//---------------------------------------------------------------------------------------
// Function:  FullScreen
// Desc:      Alters the window class styles and position for full screen apps.
//---------------------------------------------------------------------------------------

  BOOL FullScreen()
  {
    BOOL retval;

    //  Save the styles and position of the window
    GetWindowRect(&wndRect);
    wndStyles = GetWindowLong(GWL_STYLE);
    wndExStyles = GetWindowLong(GWL_EXSTYLE);

    //  Set styles and window size for fullscreen
    SetWindowLong(GWL_STYLE,WS_POPUP);
    retval = SetWindowPos(HWND_TOPMOST,0,0,GetSystemMetrics(SM_CXSCREEN),GetSystemMetrics(SM_CYSCREEN),SWP_SHOWWINDOW);
    ShowWindow(SW_SHOW);

    return retval;
  }


//---------------------------------------------------------------------------------------
// Function:  Windowed
// Desc:      Alters the window class styles, size, and position for windowed apps.  Size 
//            and position are set according to the size and position of the window before
//            the last call to FullScreen(), or the current size and position if 
//            FullScreen() has yet to be called.  Use SetWindowPos() or ResizeClient()
//            to change the size or position of the window.
//---------------------------------------------------------------------------------------

  BOOL Windowed()
  {
    BOOL retval;

    //  Restore the styles and position of the window
    SetWindowLong(GWL_STYLE,wndStyles);
    SetWindowLong(GWL_EXSTYLE,wndExStyles);
    retval = SetWindowPos(HWND_NOTOPMOST,&wndRect,SWP_DRAWFRAME);
    ShowWindow(SW_SHOW);

    return retval;
  }



//---------------------------------------------------------------------------------------
// Message Handling
//---------------------------------------------------------------------------------------

  BEGIN_MSG_MAP(CDxWindowImpl)
    MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
    MESSAGE_HANDLER(WM_SYSCOMMAND, OnSysCommand)
    MESSAGE_HANDLER(WM_ACTIVATEAPP, OnActivateApp)
  END_MSG_MAP()



//---------------------------------------------------------------------------------------
// MESSAGE_HANDLER:  OnDestroy
// Desc:             Posts the quit message.
//---------------------------------------------------------------------------------------

 LRESULT OnDestroy( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled )
  {
    PostQuitMessage(0);
    bHandled = FALSE;
    return 0;
  }


//---------------------------------------------------------------------------------------
// MESSAGE_HANDLER:  OnSysCommand
// Desc:             Stops screen savers and monitor power management.
//---------------------------------------------------------------------------------------

  LRESULT OnSysCommand( UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled )
  {
    if(((wParam & 0xFFF0) == SC_SCREENSAVE) || ((wParam & 0xFFF0) == SC_MONITORPOWER))
    {
      bHandled = true;
    }
    else
      bHandled = false;

    return 0;
  }


//---------------------------------------------------------------------------------------
// MESSAGE_HANDLER:  OnActivateApp
// Desc:             App is being activated or deactivated.  Delegates action to 
//                   ActivateApp() or DeactivateApp().
//---------------------------------------------------------------------------------------

  LRESULT OnActivateApp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL &bHandled )
  {
    if((BOOL)wParam)
      ActivateApp();
    else
      DeactivateApp();
  
    bHandled = FALSE;
    return 0;
  }



private:

  BOOL disableActivateApp;
  LONG wndStyles;
  LONG wndExStyles;
  RECT wndRect;
};

